+function($) {
  getDefaultRules = function() {
    return {
      'input[type=password]'                        : 'password',
      'input[type=text]'                            : Peteshow.randomLetters(8),
      'input[type=email], input[name*=email]'       : Peteshow.randomEmail(),
      'input[name*=number], input[type=number]'     : Peteshow.randomNumber(8),
      'input[class*=number], input[class*=decimal]' : Peteshow.randomNumber(8),
      'input[name*=date]'                           : Peteshow.randomDate(),
      'input[name*=phone]'                          : Faker.PhoneNumber.phoneNumberFormat(5),
      'input[name*=first_name]'                     : Faker.Name.firstName(),
      'input[name*=last_name]'                      : Faker.Name.lastName(),
      'input[name*=company]'                        : Faker.Company.companyName(),
      'input[name*=line1], input[name*=street]'     : Faker.Address.streetName(),
      'input[name*=line2], input[name*=suite]'      : Faker.Address.secondaryAddress(),
      'input[name*=city]'                           : Faker.Address.city(),
      'input[name*=zip], input[name*=postal]'       : Faker.Address.zipCodeFormat(0),
      'input[name*=state]'                          : Faker.Address.usState(),
      'input[name*=job_title]'                      : Faker.Company.catchPhrase(),
      'input[name*=intent]'                         : Faker.Lorem.sentence(),
      'input[name*=income], input[name*=amount]'    : Peteshow.randomNumber(4),
      'input[name*=branch], input[name*=routing]'   : '400001',
      'input[name*=card_type_cd]'                   : '001',
      'input[name*=card_number]'                    : '4111111111111111',
      'input[name*=cvv]'                            : '123'
    }
  }

  Peteshow.init = function(options) {
    _options  = $.extend(true, Peteshow.defaults, options || {})

    $div      = $('<div/>', { id: 'peteshow' })
    $toggle   = $('<a/>',   { id: 'peteshow-toggle', href: '#' }).append('<i/>')
    $tools    = $('<div/>', { id: 'peteshow-tools' })
    $commands = $('<ul/>',  { id: 'peteshow-commands' })

    $div.append($toggle)
        .append($tools.append($commands))

    $('body').append($div)

    Peteshow.commands()
  }

  Peteshow.keydown = function(e) {
    var key   = (typeof e.which == 'number') ? e.which : e.keyCode,
        code  = String.fromCharCode(e.keyCode)

    if(e.ctrlKey) code = 'ctrl_'+code
    if(e.altKey || (e.originalEvent && e.originalEvent.metaKey)) code = 'alt_'+code
    if(e.shiftKey) code = 'shift_'+code
    if($.inArray(e.keyCode, [9,16,17,18, 91, 93, 224]) != -1) return
    if(e.metaKey) return

    if(e.keyCode == 192) { // `
      $tools.toggle()
      $div.toggleClass('active')
    }

    var action  = $("[data-command='"+code+"']"),
        visible = $tools.is(':visible')

    if(action.length > 0 && visible) action.click()
  }

  Peteshow.commands = function() {
    var base =  "<li><a data-command='F' href='#' id='fill-out-forms'>Fill Out Forms</a></li>"
        base += "<li><a data-command='Q' href='#' id='fill-out-forms-and-submit'>Fill Out and Submit</a></li>"
        base += outputLocalStorage()
        base += "<li><a data-command='H' href='#' id='hide-peteshow'>Hide</a></li>"

    $div.find($commands).html(_options.commands + base)

    Peteshow.events()
  }

  Peteshow.events = function() {
    var commands = [
      [ $toggle,                          function() { $div.toggleClass('active'); $tools.toggle() } ],
      [ $('#fill-out-forms'),             function() { Peteshow.fillOutForms() } ],
      [ $('#fill-out-forms-and-submit'),  function() { Peteshow.fillOutForms(); Peteshow.submitForm() } ],
      [ $('#clear-localstorage'),         function() { clearLocalStorage() } ],
      [ $('#hide-peteshow'),              function() { $div.hide() } ]
    ]

    $.each(commands, function() {
      var command = $(this)

      $(command[0]).on('click', function() {
        command[1]()
        return false
      });
    });

    _options.events()
  }

  Peteshow.submitForm = function() {
    $(_options.form).submit()
    $('.simple_form').last().submit()
    $('form').last().submit()
  };

  Peteshow.fillOutForms = function() {
    // select
    $('select').each(randomSelectValue)

    // checkbox
    $('input[type=checkbox]').prop('checked', true)

    // radio button
    randomRadioValue()

    // force rules
    $.each(_options.force, function(element,v) {
      $(element)
        .filter(function() { return _options.ignore.indexOf(this.id) === -1 })
        .val($.isFunction(v) ? v() : v)

      if(_options.blur) $(element).blur()
    })

    // apply rules
    var rules     = $.extend(true, getDefaultRules(), _options.rules || {})
        reused    = {},
        ls        = getLocalStorage(),
        local     = (ls != null || ls != undefined) ? ls : {}

    // apply value to rule element, if visible and not in ignore list
    $.each(rules, function(element,v) {
      $(element).filter(':visible')
                .filter(function() { return _options.ignore.indexOf(this.id) === -1 })
                .val($.isFunction(v) ? v() : v)

      if(_options.blur) $(element).blur()
    })

    // special rules
    _options.special()

    // reuse values
    $.each(_options.reuse, function(element,v) {
      var url = _options.reuse[element]

      if($(element).length > 0) {
        // if element isnt in localstorage, save it in localstorage
        if(!(element in local))
          reused[element] = $(element).val()

        // if element is in localstorage and we're not on the reused url, save it in localstorage
        if((element in local) && window.location.href.indexOf(url) < 0)
          reused[element] = $(element).val()
      }
    })

    // save localstorage if found rules to reuse
    if(!$.isEmptyObject(reused)) {
      $.extend(local, reused)

      localStorage.setItem('peteshow', JSON.stringify(local))
    }

    // apply localstorage rule values if they exist and on the right page
    if(localStorage.peteshow != undefined || localStorage.peteshow != null) {
      $.each(getLocalStorage(), function(element,v) {
        var url = _options.reuse[element]

        if(window.location.href.indexOf(url) > -1)
          $(element).val(v)
      })

      // reinit menu
      Peteshow.commands()
    }
  }

  outputLocalStorage = function() {
    var base = ''
    if(localStorage.peteshow != undefined || localStorage.peteshow != null) {
      base += "<li class='list'>"
        base += "<div class='inner'>"
          $.each(getLocalStorage(), function(k,v) {
            base += '<div>' + k + '<span>' + v + '</span></div>'
          })
        base += "</div>"
      base += "</li>"

      base += "<li><a data-command='R' href='#' id='clear-localstorage'>Clear stored</a></li>"
    }
    return base
  }

  getLocalStorage = function() {
    return JSON.parse(localStorage.getItem('peteshow'))
  }

  clearLocalStorage = function() {
    localStorage.removeItem('peteshow')
    Peteshow.commands() // reinit menu to remove ls
  }

  randomSelectValue = function(i,select) {
    var options   = $(select).find('option'),
        filters   = _options.filter.toString().replace(new RegExp(',', 'g'), '|'),
        regex     = new RegExp('other|select'+(filters == '' ? '' : '|' + filters),'gi'),
        filtered  = []

    $.each(options, function(e) {
      var value = $(this).val()

      if(value.match(regex) == null && value != '')
        filtered.push(value)
    });

    var random = Math.floor(Math.random() * filtered.length)

    $(select)
      .filter(function() { return _options.ignore.indexOf(this.id) === -1 })
      .val(filtered[random]).change()
  }

  randomRadioValue = function() {
    var names = $('input:radio').map(function() {
      return $(this).attr('name')
    })

    $.unique(names)

    $.each(names, function(i, name) {
      var radios = $('input:radio[name="'+name+'"]')

      $(radios[Math.floor(Math.random() * radios.length)])
        .filter(function() { return _options.ignore.indexOf(this.id) === -1 })
        .prop('checked', true).change()
    })
  }

  $(document).keydown(Peteshow.keydown);

}(jQuery)
